! 20.07.2004 
!+
! Module: omsourceflux_mod
!
! Authors:  Christopher Brindle - MSSL
!           
!          
! Description: Combines OSW source list into an observation source list
! 
! Comments: 
! 
! $Id: module.f90,v 1.5 1998/01/13 08:39:15 gvacanti Exp $
!
! History: 
!       
!
!************************************************************************************************
module omsourceflux_mod
  use param
  use cal
  use dal
  use Errorhandling
  use sla
  use getFilterID_mod
 
  use omallocation_mod
  use omutility_mod
  use omastromcorrect_mod 
  use copytable_mod
  use omgraph_mod
  use fluxfit_mod
  use nonlinls_mod
  implicit none 

  private 

  public omsourceflux

  !*****************************
  ! Structure to define a color
  !***************************** 
   type ColourT
     integer :: mag1
     integer :: mag2
     character(20) label 
    character(20) elabel
     character(20) flabel
  end type ColourT

 


contains 
!****************************************
! The main program
!****************************************
subroutine omsourceflux

    integer, parameter               :: nFilt = 7             ! no of filters
    integer, parameter               :: nCol = 6             ! no of filters
   
    type(ColourT)                    :: instrumentalColour(nCol)
    type(ColourT)                    :: standardColour(nCol)

    type(DataSetT)                   :: oswDataSet            ! handles
    type(DataSetT)                   :: outDataSet            ! handle for output dataset
    type(TableT)                     :: srcListTable
    type(ColumnT) :: pColumn
  
    integer                          :: nosw                   ! no of osw lists
    integer                          :: i,j,k                  ! counters
 
    integer(int32) :: nRows, nColumns
      
    character(5)                 :: filtid(nFilt) = &
                             (/'WHITE','UVW2 ','UVM2 ','UVW1 ','U    ','B    ','V    '/) 
    integer(int32)              :: filt(6) =  (/UVW2, UVM2, UVW1, U, B, V /)
    integer(int32)               :: intNullValue = 10      
   
    real(double) :: f0(7)
    character(10) :: columnName
    character(10) :: instMagName(6)
    type(sourceData) :: magList

    logical(boolean) :: fitted
    real(double) :: rmsMinimum
    integer(int16) :: templateNo, noOfDegreesOfFreedom

  
    character(200) :: oldObsFile, newObsFile
    integer :: noFits
    type(fittedData) :: fit                                                      
    real(single), pointer :: tempData(:, :) => null()
  

    ! Get the constants for converting 
    ! a zero-magnitude star int a count-rate for each filter
    call GetZeroCounts(filtid, f0)


    !************************
    ! Iniitialise NullValue
    !************************
    nullValue = -real(99999)

   
    call SetUpInstrumentalMagnitudeTable(instMagName)


    !*****************************************************
    ! Open the combined observation-source list file
    !*****************************************************
    oldObsFile = trim(stringParameter("combinedsourcelist"))     
    newObsFile = trim(stringParameter("outset"))     

    ! Create the new observation data file
    call CreateNewObsSourceList(oldObsFile, newObsFile)
   

    oswDataSet = dataset(oldObsFile, READ)      

    srcListTable = Table(oswDataSet, "SRCLIST", READ)
    nRows = numberOfRows(srcListTable)
    nColumns = numberOfColumns(srcListTable)
   


    allocate(magTable(1 : nRows))
    call loadInstrumentalMags(srcListTable, instMagName, filtid, magTable)

  

!  do i =0, numberOfColumns( tab ) - 1
!    col = column( tab, i, READ )
!    write(*,*) name( col ), columnDataType( col ), units( col )
!  end do



!    call ComputeSourceFluxes(nRows, magList, f0, instMagName)


    call release(srcListTable)
    call release(oswDataSet)   

    ! Add the new flux columns
    call AddFluxColumns(magTable, newObsFile)

    tempData => allocate2dSingle(1, size(magTable), 1, 6, "tempData")
    do j = 1, 6
        tempdata(:, j) = magTable(:)%flux(j)
    end do

 
    if (associated(magTable)) deallocate(magTable)
   
    if (associated(templateData)) deallocate(templateData)

    deallocate(tempData)
end subroutine omsourceflux
!******************************************
! CreateNewObsSourceList
! create the output observation source list
!******************************************
subroutine CreateNewObsSourceList(oldFile, newFile)
    character(*), intent(in) :: oldFile
    character(*), intent(out) :: newFile
    integer :: i
    type(BLOCKT) :: blk 

    type(DataSetT) :: oldDataSet, newDataSet
    
    call copyDataSet(oldFile, newFile)

    newDataSet = dataSet(newFile, READ)
    do i = 0, numberOfBlocks(newDataSet) - 1
        blk = block(newDataSet, i, MODIFY)
        call addComment(blk, "Created and modified by omsourceflux")
    end do
    call release(newDataSet)
       
end subroutine CreateNewObsSourceList

!******************************************************
! subroutine SetUpStandardColourTable
! Initialises the array with the possible colours
! Letters refer to letter codes for filters
! ie V=V, U=U, B=B, W=White, G=Grism2, X=UVW1,
!   Y=UVM2, Z=UVW2 & H=Grism1
!******************************************************
subroutine SetUpInstrumentalMagnitudeTable(instMagName)

    character(*), intent(out), dimension(:) :: instMagName
  
    instMagName(1) = "UVW2"
    instMagName(2) = "UVM2"
    instMagName(3) = "UVW1"
  
   
    instMagName(4) = "U_INS"
    instMagName(5) = "B_INS"
    instMagName(6) = "V_INS"
    

end subroutine SetUpInstrumentalMagnitudeTable

!****************************************************
!
!***************************************************
subroutine loadSourceMags(instMagName, f0, rowNo, magTable, magList)
  
    character(*), intent(in) :: instMagName(:)
    real(double), intent(in) :: f0(:)
    integer, intent(in) :: rowNo
    type(sourceData), pointer :: magTable(:)
    type(sourceData), intent(out) :: magList
    integer :: i
 
    real(double) :: error
    type(sourceData) :: tempList
    
    magList = tempList
    do i = 1, 6
        
        if (magTable(rowNo)%mag(i) .lt. 0.0 .or.  magTable(rowNo)%mag(i) .gt. 30.0) cycle
        magList%mag(i) = magTable(rowNo)%mag(i)
        magList%magErr(i) = magTable(rowNo)%magErr(i)
        magList%counts(i) = f0(i) * 10.0_double ** (-0.4_double *  magList%mag(i))
        error = 0.92104 * magList%magErr(i) * magList%counts(i)  
        magList%weight(i) = 1.0_double / (error * error)
        magList%error(i) = error
                
    end do

   
end subroutine loadSourceMags        

!****************************************************
!
!***************************************************
subroutine loadInstrumentalMags(srcListTable, instMagName, filterList, magTable)
    type(TableT), intent(in) :: srcListTable
    character(*), intent(in) :: instMagName(:), filterList(:)
    type(sourceData), intent(out) :: magTable(:)
    integer :: i, j, nRows
    type(ColumnT) :: pColumn
    character(10) :: columnName
    real(single), pointer :: tempData(:) => null()
    integer :: m = 0
    character(4) :: filter(1 : 6) = (/"    ", "    ", "    ", "    ", "    ", "    "/)
    write(errstr, '(a)') "Loading source data"
    call message(AppMsg, Verbose)
    


    nRows = numberOfRows(srcListTable) ! No of rows in source-list table
  
    tempData => allocateSingle(1, nRows, "tempData")
 
    do i = 1, NumberOfColumns(srcListTable) ! Loop through all the columns
    
        pColumn = column(srcListTable, i - 1, READ)
        columnName = name(pColumn)

        do j = 1, 6
            if (trim(ColumnName) .eq. trim(instMagName(j))) then
                call ReadReal32Column(srcListtable, columnName, nRows, 0, tempdata)
                magTable(:)%mag(j) = dble(tempdata)
                columnName = "ERR_" // columnName
                call ReadReal32Column(srcListtable, columnName, nRows, 0, tempdata)
                magTable(:)%magErr(j) = dble(tempdata)
                 
                m = m + 1
                filter(m) = trim(filterList(j))
                exit               
            end if
        end do
        call release(pColumn)
    end do
    magTable%noFilters = m
    do i = 1, m
        magTable%filter(i) = filter(i)
    end do
    deallocate(tempData)
    nullify(tempdata)   
end subroutine loadInstrumentalMags        




!***********************************************************************
!
!
!**********************************************************************
subroutine ComputeSourceFluxes(nRows, magList, f0, instMagName)
    integer, intent(in) :: nRows    
    type(sourceData), intent(inout) :: magList
    real(double), intent(in) :: f0(:)
    character(*), intent(in) :: instMagName(:) ! , filterList(:)

    character(200) :: fileName(3) = (/"/sschome/ssco2/sasdevel/packages/omsourceflux/test/res_counts_om_f02_numb.txt", &
                                      "/sschome/ssco2/sasdevel/packages/omsourceflux/test/res_flux_om_f02_numb.txt  ", &
                                      "/sschome/ssco2/sasdevel/packages/omsourceflux/test/res_flux_om_f02_text.txt  "/)
  
    integer :: noFits
    type(fittedData) :: fit                                                      
    real(single), pointer :: tempData(:, :) => null()
    logical(boolean) :: fitted
    real(double) :: rmsMinimum
    integer(int16) :: templateNo, noOfDegreesOfFreedom
    integer :: i, j  
    
 

    ! Load the stellar templates
    call LoadTemplateData(templateData)

    open(unit=10, status="UNKNOWN", file="fittedspectra.dat")
    write(10, '(a12, a10, a10, a5)') "Source no", & 
           "Temp No", "Rms Res", "MU"


    call pgbegin(0, trim(stringParameter("plotfile")) // '/VC', 1, 1)
  
    
   
    noFits = 0


    do i = 1, nRows

        magTable(i)%flux(1 : 6) = nullValue
        magTable(i)%fluxErr(1 : 6) = nullValue
  
        call loadSourceMags(instMagName, f0, i, magTable, magList)       
        call FitSpectra(magList, templateData, fitted, fit)
        if (fitted) then
            call updateLogFile(i, rmsMinimum, templateNo,  noOfDegreesOfFreedom)
            
            ! Plot the fit
            call PlotFit(i, fit)


            magTable(i)%flux = fit%flux
            magTable(i)%fluxErr = fit%fluxErr
            noFits = noFits + 1

        else

            magTable(i)%flux = fit%flux
            magTable(i)%fluxErr = fit%fluxErr
     

        end if



    end do
    call pgclos
    write(10, '(a, i5, a, i5, a)') "Total no of fitted spectra=", noFits, " (no of sources=", nRows, ")"
    
  
    write(errstr, '(a, i5, a, i5, a)') "Total no of fitted spectra=", noFits, " (no of sources=", nRows, ")"
   
    call message(AppMsg, Verbose)
    write(10, *) errstr          
    close(10)
   

    ! Add the new flux columns
    call AddFluxColumns(magTable,  trim(stringParameter("outset")) )

    tempData => allocate2dSingle(1, size(magTable), 1, 6, "tempData")
    do j = 1, 6
        tempdata(:, j) = magTable(:)%flux(j)
    end do

end subroutine ComputeSourceFluxes

end module omsourceflux_mod









